#include "framebuffer.h"
#include "math.h"
#include <iostream>
#include "scene.h"
#include "v3.h"
#include <iostream>

using namespace std;

FrameBuffer::FrameBuffer(int u0, int v0,
	int _w, int _h) : Fl_Gl_Window(u0, v0, _w, _h, 0) {

	w = _w;
	h = _h;
	pix = new unsigned int[w*h];

}


void FrameBuffer::draw() {

	glDrawPixels(w, h, GL_RGBA, GL_UNSIGNED_BYTE, pix);
}

int FrameBuffer::handle(int event) {

	switch (event)
	{
	case FL_KEYBOARD: {
		KeyboardHandle();
		return 0;
	}
	default:
		break;
	}
	return 0;
}

void FrameBuffer::KeyboardHandle() {

	float tstep = 3.0f;
	float rstep = 3.0f;
	int key = Fl::event_key();
	switch (key) {
	case FL_Left: {
		cerr << "INFO: pressed left" << endl;
		break;
	}
	case FL_Right: {
		break;
	}
	case 'a': {
		cerr << "INFO: pressed a" << endl;
		break;
	}
	default:
		cerr << "INFO: do not understand keypress" << endl;
	}
}

void FrameBuffer::Set(V3 uv, unsigned int color) {

	SetGuarded((int)uv[0], (int)uv[1], color);

}

void FrameBuffer::Set(int u, int v, unsigned int color) {

	pix[(h - 1 - v)*w + u] = color;

}

void FrameBuffer::SetGuarded(int u, int v, unsigned int color) {

	if (u < 0 || u >= w || v < 0 || v >= h)
		return;
	Set(u, v, color);

}

void FrameBuffer::SetBGR(unsigned int bgr) {

	for (int uv = 0; uv < w*h; uv++)
		pix[uv] = bgr;

}

void FrameBuffer::Draw2DRectangle(int u0, int v0, int u1, int v1, unsigned int color) {

	for (int v = v0; v <= v1; v++) {
		for (int u = u0; u <= u1; u++) {
			SetGuarded(u, v, color);
		}
	}



}

void FrameBuffer::DrawCircle(int u0, int v0, float r, unsigned int color) {

	int left = (int)(u0 - r);
	if (left < 0)
		left = 0;
	if (left > w - 1)
		return;
	int right = (int)(u0 + r);
	if (right < 0)
		return;
	if (right > w - 1)
		right = w-1;

	int top = (int)(v0 - r);
	if (top < 0)
		top = 0;
	if (top > h - 1)
		return;
	int bottom = (int)(v0 + r);
	if (bottom < 0)
		return;
	if (bottom > h - 1)
		bottom = h - 1;

	V3 cc(.5f + (float)u0, .5f + (float)v0, 0.0f);
	for (int v = top; v <= bottom; v++) {
		for (int u = left; u <= right; u++) {
			V3 cpc(.5f + (float)u, .5f + (float)v, 0.0f);
			if ((cpc - cc).Length() <= r)
				Set(u, v, color);
		}
	}


}

void FrameBuffer::Draw2DSegment(V3 p0, V3 p1, unsigned int color) {

	float du = fabsf(p0[0] - p1[0]);
	float dv = fabsf(p0[1] - p1[1]);
	int stepsN;
	if (du > dv) {
		stepsN = (int)(du + 2);
	}
	else {
		stepsN = (int)(dv + 2);
	}

	V3 duv = (p1 - p0) / (float)stepsN;
	for (int stepi = 0; stepi < stepsN; stepi++) {
		V3 currp = p0 + duv * (float) stepi;
		Set(currp, color);
	}

}
